<?php
/**
 * Base class for any API module. This class should be extended in the module itself.
 * 
 * @copyright 2014 FreshRealm
 * @author Ondrej Nebesky <ondrej@freshrealm.co>
 * @author John Grogg <grogg@freshrealm.co>
 * @version 0.9
 * @package FreshRest
 */
class FrApiBaseModule extends CWebModule {

    public $version = 1;
    public $format = 'json';

    /**
     * Attribute used to find the last update time of active record in database.
     * This attribue is compared to "timestamp" parameter in api for syncing only
     * changed records with client.
     * @var string 
     */
    public $lastUpdateAttribute = 'update_time';

    /**
     * @var string string the id of the default controller for this module.
     */
    public $defaultController = 'default';
    
    /**
     * Url used to construct links for api that include authentication token
     * @var type 
     */
    public $baseUrl = null;
    public $schema = 'http';

    /**
     * Name of the authentication table that will be created the first time
     * this module is being used. Set to empty string or null to disable builtin
     * authentication
     * @var string 
     */
    public $authTableName = "fr_api_device";
    public $authModelClass = "FrAuthModel";
    public $myAuthenticatedModelClass = '';
    public $myAuthenticatedModelPasswordField = '';
    public $authCacheDuration = 3600; // one hour
    protected $authData;
    protected $authModel;
    protected $authenticatedModel;
    public $errorCodes = array(
        200 => 'OK',
        400 => 'Bad Request',
        401 => 'Access Denied',
        403 => 'Token key is not valid',
        404 => 'Resource Not Found',
        500 => 'Internal Server Error',
        501 => 'Not Implemented'
    );

    protected function getAuthData() {
        if ($this->authData != null) {
            return $this->authData;
        }
        $modelClass = $this->authModelClass;
        $this->authData = array(
            'key' => isset($_GET['key']) ? $_GET['key'] : null,
            'ipAddress' => $_SERVER['REMOTE_ADDR'],
            'method' => $_SERVER['REQUEST_METHOD'],
                //'authModel' => new $modelClass($this)
        );
        $this->authModel = new $modelClass($this, $this->authData);
        return $this->authData;
    }

    public function getAuthModel() {
        if ($this->authModel != null) {
            return $this->authModel;
        }
        $this->getAuthData();
        return $this->authModel;
    }

    /**
     * Return authenticated active record
     * @return CActiveRecord
     */
    public function getAuthenticatedModel() {
        $auth = $this->getAuthModel();
        return $auth->getModel();
    }

    public function getAuthKey() {
        $data = $this->getAuthData();
        return $data['key'];
    }

    public function getIpAddress() {
        $data = $this->getAuthData();
        return $data['ipAddress'];
    }

    public function init() {
        $id = $this->id;
        $this->setImport(
                array(
                    "application.modules.$id.components.*",
                    "application.modules.$id.controllers.*",
                    "application.modules.$id.models.*",
                )
        );
        $this->installAuthSchema();
        
        // construct default url
        if ($this->baseUrl == null){
            $this->baseUrl = Yii::app()->createAbsoluteUrl('/' . $this->getId(), array(), $this->schema);
        }
        if (substr($this->baseUrl,0,4) != 'http'){
            $this->baseUrl = $this->schema . '://' . $this->baseUrl;
        }
    }
    
    public function getVersion(){
        return $this->version;
    }

    /**
     * Check for existence of databse table that stores temporary auth tokens
     * and create it if it doesn't exist.
     * @return null
     */
    public function installAuthSchema() {
        if (strlen($this->authTableName) == 0) {
            return;
        }
        $tableSchema = Yii::app()->db->schema->getTable($this->authTableName);
        if ($tableSchema == null) {
            $createSql = Yii::app()->db->schema->createTable($this->authTableName, array(
                'id' => 'int(11) unsigned not null auto_increment primary key',
                'token' => 'varchar(63)',
                'ip_address' => 'varchar(45)',
                'update_time' => 'datetime',
                'connected_type' => 'varchar(45)',
                'connected_id' => 'string'
            ));
            Yii::app()->db->createCommand($createSql)->execute();
        }
    }

    public function getUrl($resource = '') {
        return$this->baseUrl . '/' . $resource;
    }


}
